︠4af9e940-47f5-4a16-a774-f53e0f71cbb8︠
%cython
#clib m

cdef extern from "math.h":
    float expf(float x)

# This function computes the dissonance of two partials
# at frequencies f1 and f2 (f1 < f2) with amplitudes of
# a1 and a2, respectively.
# This is using the Plomp-Levelt dissonance curve, as described in
# http://eceserv0.ece.wisc.edu/~sethares/consemi.html
def diss(float f1, float f2, float a1, float a2):
    cdef float s = (f2 - f1)*0.24/(0.0207*f1 + 18.96)
    return 5*a1*a2*(expf(-3.51*s) - expf(-5.75*s))
︡30058705-d1b5-4e93-8d97-d91d9e22c996︡︡
︠b03fbbce-b4b1-481c-9337-5df7745b3753︠
# discrete dissonance matrices
from itertools import product

# Bohlen Pierce just and et, with odd harmonic partials
# the outcome of this algorithm heavily depends on the parameters, see examples below
def compute_ddm(just_scale=True, number_harmonics=7, square_wave=True, base_freq=256, tritaves=2):
    # just_scale       = True  # else equal temperament
    # number_harmonics = 10    # only odd harmonics are counted
    # square_wave      = True  # else triangle
    # base_freq        = 256   # affects dissonace (plomb/levelt curve width)

    n   = tritaves*13 + 1        # range of triads
    ddm = matrix(RR, n, n)   # dissonance values

    # fill in the frequencies of the partials here
    par = [base_freq*(2*i + 1) for i in range(number_harmonics)]

    # with the respective amplitudes
    if square_wave:
        amp  = [1.0/i for i in range(1, len(par) + 1)]
    else: # triangle
        amp  = [1.0/(i**2) for i in range(1, len(par) + 1)]

    # steps, as frequency ratios
    if just_scale:
        steps = []
        _steps = [1, 27/25, 25/21, 9/7, 7/5, 75/49, 5/3, 9/5, 49/25, 15/7, 7/3, 63/25, 25/9]
        for i in range(tritaves):
            steps += [s*3**i for s in _steps]
        steps += [3**tritaves]
    else:
        steps = [float(3**(i/13)) for i in range(n)]

    pa = zip(par, amp)

    for i, j in product(range(n), repeat=2): # all triads 0, i, j
        pa2 = (pa + [(float(p*steps[i]),a) for (p,a) in pa]
                  + [(float(p*steps[j]),a) for (p,a) in pa])
        pa2.sort()
        ddm[i,j] = 0 # consonance is the absence of dissonance
        for k in range(0, len(pa2) - 1):     #  lower partial
            for l in range(k + 1, len(pa2)): # higher partial
                if (pa2[k][0] == pa2[l][0]):
                    continue
                # add up dissonance for all pairs of partials of the three notes
                ddm[i,j] += diss(pa2[k][0], pa2[l][0], pa2[k][1], pa2[l][1])
    
    return ddm
︡3fa396a9-1e78-4107-a6b8-993964de218f︡︡
︠eb7ba22a-b174-4c3f-bb14-1ae1db4de00c︠
# note that the heat map values can only be interpreted
# relatively, not absolutely.
# i.e., looking at one matrix, bright/yellow is consonant
# while dark/red is more dissonant.
# but don't compare the color between one matrix and another,
# as they may be scaled differently.
#
# one could stack the matrices and plot them together, though
#
# consonance is higher for fewer harmonics with lower amplitudes
# also, the width of the Plomp/Levelt curve doesn't scale lineary,
# so the choice of the base tone frequency is important.
#
# a triad consists of three tones and denoted (i,j,k)
# where i,j & k are integers that map the steps in chromatic BP
#
# to get 2-dimensional data, we fix i at 0, the base tone of the scale
# j and k are then allowed to move between 0 and 3*13, if you choose 3 tritaves
# the value of the matrix at (j,k) then gives the dissonance of the triad (0,j,k)
#
# note that the matrix is symmetric, because the triads (0,j,k) and (0,k,j) are identical
#
# on the diagonal and the 0 row and column, two of the three tones are identical
# these are not proper triads and thus more consonant
︡8796a306-d89e-4be0-8b35-f0ccc41a8358︡︡
︠a5812b99-a1fe-4cb1-9011-f6e25ac4be9c︠
# compute dissonance heat map for triads
#  with options:
#  just tuning
#  9 harmonics of square wave
#  base tone of triads is middle c
#  all triads, that fit in 3 tritaves above base tone
ddm  = compute_ddm(True, 9, True, 261.6, 3)
mp = matrix_plot(matrix(ddm.rows()[::-1]), cmap='hot')
mp.save("bla.png")
︡95037fd0-8546-4f43-930c-091c5326c949︡︡
︠ed6bd37d-7d81-4849-8a83-b2b615759b80︠
# compute dissonance heat map for triads
#  with options:
#  equal temperament
#  9 harmonics of square wave
#  base tone of triads is middle c
#  all triads, that fit in 3 tritaves above base tone
ddm  = compute_ddm(False, 9, True, 261.6, 3)
mp = matrix_plot(matrix(ddm.rows()[::-1]), cmap='hot')
mp.save("bla.png")
︡03fc28aa-5c30-40e0-a515-0528c9ef8adc︡︡
︠e8fc5d4b-e605-48e8-a741-2272e1740d5b︠
# compute dissonance heat map for triads
#  with options:
#  equal temperament
#  9 harmonics of triangle wave
#  base tone of triads is middle c
#  all triads, that fit in 3 tritaves above base tone
ddm  = compute_ddm(True, 7, True, 261.6, 3)
mp = matrix_plot(matrix(ddm.rows()[::-1]), cmap='hot')
mp.save("bla.png")
︡9e62d980-fd23-4b87-bf60-40078ab37551︡︡
︠240f49ec-26af-49cf-9fcc-0ae61ebea069︠
# compute dissonance heat map for triads
#  with options:
#  equal temperament
#  9 harmonics of triangle wave
#  base tone of triads is at two octaves below middle c
#  all triads, that fit in 3 tritaves above base tone
ddm  = compute_ddm(True, 7, True, 261.6/4, 3)
mp = matrix_plot(matrix(ddm.rows()[::-1]), cmap='hot')
mp.save("bla.png")
︡ee2ddd9d-d506-4957-9f57-dfccb85f852b︡︡
︠fad72d4c-afa6-4d52-8983-4c699c8b694f︠
# compute dissonance heat map for triads
#  with options:
#  equal temperament
#  9 harmonics of triangle wave
#  base tone of triads is at two octaves above middle c
#  all triads, that fit in 3 tritaves above base tone
ddm  = compute_ddm(True, 7, True, 261.6*4, 3)
mp = matrix_plot(matrix(ddm.rows()[::-1]), cmap='hot')
mp.save("bla.png")
︡fcea8fae-8c66-41bb-bbe9-a49e32045b61︡︡
︠2cc2a560-e4d4-427d-bb0b-1562c88f1629︠
# which steps are used in the most consonant way?
#  the step ranked last here could be omitted in a 
#  12-key layout.

n = 2*13 + 1
rank = [0.0]*n

for i in range(0, n):
    for j in range(0,n):
        rank[i] += ddm[i,j]
        
ranking = []
for i in range(0, 13):
    ranking.append((rank[i], i))
ranking.sort()
for r, i in ranking:
    j = (i + 13)
    print str(i).rjust(2), str(r)[:5], "\t", 
    print str(j).rjust(2), str(rank[j])[:5], "\t",
    print str(r + rank[j])[:6]
︡5f25fbde-8c70-41f3-9800-4400421ff805︡{"stdout": "0 4.580 \t13 4.524 \t9.1047\n10 5.512 \t23 4.144 \t9.6573\n 4 5.780 \t17 5.823 \t11.603\n 8 5.816 \t21 4.654 \t10.470\n 6 5.823 \t19 4.312 \t10.135\n 3 6.278 \t16 5.898 \t12.177\n 9 6.339 \t22 5.300 \t11.640\n11 6.625 \t24 5.911 \t12.537\n 7 6.713 \t20 6.703 \t13.416\n 5 7.696 \t18 7.231 \t14.927\n 2 8.875 \t15 6.594 \t15.469\n12 10.31 \t25 5.282 \t15.601\n 1 24.92 \t14 9.769 \t34.693"}︡
︠16dd2d3d-8510-4f75-a1fa-f8a73c4e87b9︠
# what proper triads are most/least consonant?

ddl = [(ddm[i,j],i,j) for (i,j) in product(range(n),repeat=2) if (0 < i) and (i < j)]
ddl.sort()
print "All", len(ddl), "proper triads in 9:1 region,"
print "sorted by ascending dissonance."
print

for (v,i,j) in ddl:
    print 0, str(i).rjust(2), str(j).rjust(2), ":", str(v)[:6]
︡97d6ced2-a24d-41d8-a120-96dc1504df74︡{"stdout": "WARNING: Output truncated!"}︡{"html": "<a target='_new' href='/home/admin/13/cells/5/full_output.txt' class='file_link'>full_output.txt</a>"}︡{"stdout": "All 325 proper triads in 9:1 region,\nsorted by ascending dissonance.\n\n0 13 26 : 0.0020\n0 13 23 : 0.0234\n0 10 23 : 0.0262\n0 23 26 : 0.0308\n0 13 19 : 0.0317\n0 19 23 : 0.0326\n0 13 21 : 0.0335\n0  6 19 : 0.0362\n0  4 23 : 0.0374\n0  8 21 : 0.0382\n0  3 26 : 0.0476\n0 19 21 : 0.0594\n0 10 13 : 0.0597\n0 19 26 : 0.0611\n0 21 23 : 0.0655\n0 10 26 : 0.0679\n0 16 26 : 0.0680\n0  3 13 : 0.0723\n0  7 26 : 0.0744\n0 19 25 : 0.0758\n0 22 26 : 0.0767\n0 19 22 : 0.0771\n0  4 26 : 0.0774\n0 13 16 : 0.0788\n0  6 25 : 0.0812\n0 21 26 : 0.0833\n0  3 19 : 0.0843\n0  4 25 : 0.0858\n0 17 23 : 0.0910\n0 10 19 : 0.0929\n0  6 10 : 0.0930\n0  3 22 : 0.0934\n0  3 16 : 0.0936\n0  4 10 : 0.0938\n0  6 23 : 0.0942\n0 13 17 : 0.0947\n0 21 24 : 0.0951\n0  9 19 : 0.0957\n0 10 21 : 0.0967\n0 11 19 : 0.0983\n0 17 21 : 0.0986\n0  8 19 : 0.0987\n0  4 17 : 0.0989\n0  5 26 : 0.0994\n0  6 13 : 0.1015\n0  4 21 : 0.1018\n0  6 21 : 0.1022\n0  4  8 : 0.1025\n0  8 16 : 0.1035\n0 11 21 : 0.1047\n0  7 13 : 0.1052\n0 23 25 : 0.1052\n0  9 13 : 0.1053\n0 15 23 : 0.1061\n0  4 13 : 0.1062\n0  8 23 : 0.1072\n\n...\n\n0  9 12 : 0.3434\n0  2 16 : 0.3526\n0 12 24 : 0.3532\n0  8 14 : 0.3589\n0 20 21 : 0.3590\n0 12 22 : 0.3612\n0  2 12 : 0.3634\n0 25 26 : 0.3652\n0  7 12 : 0.3684\n0 18 19 : 0.3796\n0 14 18 : 0.3843\n0 23 24 : 0.3902\n0  2 21 : 0.4033\n0  5 12 : 0.4045\n0 16 17 : 0.4083\n0 17 18 : 0.4133\n0  5 24 : 0.4212\n0  2 14 : 0.4413\n0 12 18 : 0.4424\n0 19 20 : 0.4483\n0 12 14 : 0.4648\n0  8  9 : 0.4786\n0 14 15 : 0.4855\n0  9 10 : 0.4948\n0 15 16 : 0.5269\n0 11 12 : 0.5357\n0  7  8 : 0.5443\n0 10 11 : 0.5811\n0  4  5 : 0.5959\n0  5  6 : 0.6131\n0 13 14 : 0.6167\n0  3  4 : 0.6220\n0 12 13 : 0.6490\n0  6  7 : 0.6711\n0  1 26 : 0.7826\n0  1 22 : 0.7869\n0  1 23 : 0.7958\n0  1 24 : 0.7967\n0  1 21 : 0.8066\n0  1 25 : 0.8126\n0  1 20 : 0.8251\n0  1  9 : 0.8328\n0  1 17 : 0.8360\n0  1 11 : 0.8390\n0  1 10 : 0.8422\n0  1  7 : 0.8440\n0  1 19 : 0.8462\n0  1  4 : 0.8496\n0  2  3 : 0.8541\n0  1 16 : 0.8611\n0  1  6 : 0.8706\n0  1  8 : 0.8777\n0  1 18 : 0.9031\n0  1  3 : 0.9085\n0  1 15 : 0.9154\n0  1 14 : 0.9239\n0  1  5 : 0.9355\n0  1 13 : 0.9356\n0  1 12 : 1.0073\n0  1  2 : 1.3379"}︡
︠071ae58b-a2df-4b78-aff2-11e93df12c66︠
# the circle of fifths/fourths is the unique generator
# for the chromatic 12 tone system, (besides the half-tone)

# but 13 is prime, so any stepsize is suitable, which one to choose
# for "related" tones

# tune violine strings in 7:5, i.e. 4 steps
step = 4# et half-steps

gd = {}
for i in range(13):
    #print i, (i*step)%13
    gd[(i*step)%13] = [((i+1)*step)%13]
    
circ = Graph(gd)
circ.plot().show()
circ.plot(layout="circular").show()
︡45080c55-6340-4403-8832-84a06dbea4b6︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡{"html": "<font color='black'><img src='cell://sage1.png'></font>"}︡
